@@ -2,11 +2,12 @@ from .univariate_selection import compute_univariate_preselection
 from .univariate_selection import get_preselected_predictors
 from .univariate_selection import compute_correlations
 
-from .models import LogisticRegressionModel
+from .models import LogisticRegressionModel, LinearRegressionModel
 from .forward_selection import ForwardFeatureSelection
 
 __all__ = ['compute_univariate_preselection',
            'get_preselected_predictors',
            'compute_correlations',
            'LogisticRegressionModel',
+           'LinearRegressionModel',
            'ForwardFeatureSelection']
