@@ -1,19 +1,4 @@
-"""
-This class implements the Python Prediction's way of dealing with
-categorical data preprocessing. There are three steps involved here:
 
-- An optional regrouping of the different categories based on category size
-  and significance of the category w.r.t. the target
-- Missing value replacement with the additional category ``Missing``
-- Change of dtype to ``category`` (could potentially lead to memory
-  optimization)
-
-Authors:
-
-- Geert Verstraeten (methodology)
-- Jan Benisek (implementation)
-- Matthias Roels (implementation)
-"""
 # standard lib imports
 import re
 from typing import Optional
@@ -29,42 +14,70 @@ from sklearn.exceptions import NotFittedError
 
 log = logging.getLogger(__name__)
 
-
 class CategoricalDataProcessor(BaseEstimator):
-    """
-    Regroups the categories of categorical variables based on significance
+    """Regroups the categories of categorical variables based on significance
     with target variable.
 
+    This class implements the Python Prediction's way of dealing with
+    categorical data preprocessing. There are three steps involved:
+
+        - An optional regrouping of the different categories based on category size
+          and significance of the category w.r.t. the target.
+            - For a given categorical variable, all categories below the (weighted)
+              category size threshold are put into a rest category (by default ``Other``)
+            - The remaining categories are subject to a statistical test, if there is
+              sufficient dependence with the target variable compared to all other categories,
+              the category is kept as-is, otherwise it is also put into the rest category
+            - Beware: one can force categories to be kept, and if no single category passes
+              the statistical test, the categorical variable is left unprocessed altogether
+        - Missing value replacement with the additional category ``Missing``.
+        - Change of dtype to ``category`` (could potentially lead to memory
+          optimization).
+
+    See the README of the GitHub repository for more methodological background information.
+
     Attributes
     ----------
     category_size_threshold : int
-        minimal size of a category to keep it as a separate category
+        All categories with a size (corrected for incidence if applicable)
+        in the training set above this threshold are kept as a separate category,
+        if statistical significance w.r.t. target is detected. Remaining
+        categories are converted into ``Other`` (or else, cf. regroup_name).
     forced_categories : dict
-        Map to prevent certain categories from being group into ``Other``
-        for each colum - dict of the form ``{col:[forced vars]}``.
+        Map to prevent certain categories from being grouped into ``Other``
+        for each column - dict of the form ``{col:[forced vars]}``.
     keep_missing : bool
-        Whether or not to keep missing as a separate category
+        Whether or not to keep missing as a separate category.
+    model_type : str
+        Model type (``classification`` or ``regression``).
     p_value_threshold : float
         Significance threshold for regrouping.
     regroup : bool
-        Whether or not to regroup categories
+        Whether or not to regroup categories.
     regroup_name : str
         New name of the non-significant regrouped variables
     scale_contingency_table : bool
-        Whether contingency table should be scaled before chi^2.'
+        Whether contingency table should be scaled before chi^2.
     """
 
-    valid_keys = ["regroup", "regroup_name", "keep_missing",
+    valid_keys = ["model_type", "regroup", "regroup_name", "keep_missing",
                   "category_size_threshold", "p_value_threshold",
                   "scale_contingency_table", "forced_categories"]
 
-    def __init__(self, regroup: bool = True, regroup_name: str = "Other",
-                 keep_missing: bool = True,
-                 category_size_threshold: int = 5,
-                 p_value_threshold: float = 0.001,
-                 scale_contingency_table: bool = True,
-                 forced_categories: dict = {}):
+    def __init__(self,
+                 model_type: str="classification",
+                 regroup: bool=True,
+                 regroup_name: str="Other",
+                 keep_missing: bool=True,
+                 category_size_threshold: int=5,
+                 p_value_threshold: float=0.001,
+                 scale_contingency_table: bool=True,
+                 forced_categories: dict={}):
+        
+        if model_type not in ["classification", "regression"]:
+            raise ValueError("An unexpected model_type was provided. A valid model_type is either 'classification' or 'regression'.")
 
+        self.model_type = model_type
         self.regroup = regroup
         self.regroup_name = regroup_name
         self.keep_missing = keep_missing
@@ -77,13 +90,13 @@ class CategoricalDataProcessor(BaseEstimator):
         self._cleaned_categories_by_column = {}
 
     def attributes_to_dict(self) -> dict:
-        """Return the attributes of CategoricalDataProcessor as a dictionary
+        """Return the attributes of CategoricalDataProcessor as a dictionary.
 
         Returns
         -------
         dict
             Contains the attributes of CategoricalDataProcessor instance with
-            the attribute name as key
+            the attribute name as key.
         """
         params = self.get_params()
 
@@ -107,7 +120,7 @@ class CategoricalDataProcessor(BaseEstimator):
         Raises
         ------
         ValueError
-            In case _cleaned_categories_by_column is not of type dict
+            In case _cleaned_categories_by_column is not of type dict.
         """
         _fitted_output = params.pop("_cleaned_categories_by_column", {})
 
@@ -131,17 +144,17 @@ class CategoricalDataProcessor(BaseEstimator):
 
     def fit(self, data: pd.DataFrame, column_names: list,
             target_column: str):
-        """Fit the CategoricalDataProcessor
+        """Fit the CategoricalDataProcessor.
 
         Parameters
         ----------
         data : pd.DataFrame
-            data used to compute the mapping to encode the categorical
+            Data used to compute the mapping to encode the categorical
             variables with.
         column_names : list
-            Columns of data to be processed
+            Columns of data to be processed.
         target_column : str
-            Column name of the target
+            Column name of the target.
         """
 
         if not self.regroup:
@@ -168,8 +181,9 @@ class CategoricalDataProcessor(BaseEstimator):
 
     def _fit_column(self, data: pd.DataFrame, column_name: str,
                     target_column) -> set:
-        """Compute which categories to regroup into "Other" for a particular
-        column
+        """Compute which categories to regroup into "Other"
+        for a particular column, and return those that need
+        to be kept as-is.
 
         Parameters
         ----------
@@ -181,15 +195,20 @@ class CategoricalDataProcessor(BaseEstimator):
         Returns
         -------
         list
-            list of categories to combine into a category "Other"
+            List of categories to combine into a category "Other".
         """
+        model_type = self.model_type
+
         if len(data[column_name].unique()) == 1:
             log.warning(f"Predictor {column_name} is constant"
                         " and will be ignored in computation.")
             return set(data[column_name].unique())
 
         y = data[target_column]
-        incidence = y.mean()
+        if model_type == "classification":
+            incidence = y.mean()
+        else:
+            incidence = None
 
         combined_categories = set()
 
@@ -201,13 +220,14 @@ class CategoricalDataProcessor(BaseEstimator):
         unique_categories = list(X.unique())
 
         # do not merge categories in case of dummies, i.e. 0 and 1
-        # (and possibly "Missings")
+        # (and possibly "Missing")
         if (len(unique_categories) == 2
             or (len(unique_categories) == 3
                 and "Missing" in unique_categories)):
             return set(unique_categories)
 
         # get small categories and add them to the merged category list
+        # does not apply incidence factor when model_type = "regression"
         small_categories = (CategoricalDataProcessor
                             ._get_small_categories(
                                 X,
@@ -221,6 +241,7 @@ class CategoricalDataProcessor(BaseEstimator):
 
             pval = (CategoricalDataProcessor
                     ._compute_p_value(X, y, category,
+                                      model_type,
                                       self.scale_contingency_table))
 
             # if not significant, add it to the list
@@ -235,20 +256,20 @@ class CategoricalDataProcessor(BaseEstimator):
 
     def transform(self, data: pd.DataFrame,
                   column_names: list) -> pd.DataFrame:
-        """Transform the data
+        """Transform the data.
 
         Parameters
         ----------
         data : pd.DataFrame
-            data used to compute the mapping to encode the categorical
+            Data used to compute the mapping to encode the categorical
             variables with.
         column_names : list
-            Columns of data to be processed
+            Columns of data to be processed.
 
         Returns
         -------
         pd.DataFrame
-            data with additional transformed variables
+            Data with additional transformed variables.
         """
 
         if self.regroup and len(self._cleaned_categories_by_column) == 0:
@@ -272,19 +293,19 @@ class CategoricalDataProcessor(BaseEstimator):
                           column_name: str) -> pd.DataFrame:
         """Given a DataFrame, a column name and a list of categories to
         combine, create an additional column which combines these categories
-        into "Other"
+        into "Other".
 
         Parameters
         ----------
         data : pd.DataFrame
-            Original data to be tranformed
+            Original data to be transformed.
         column_name : str
-            name of the column to transform
+            Name of the column to transform.
 
         Returns
         -------
         pd.DataFrame
-            original DataFrame with an added processed column
+            Original DataFrame with an added processed column.
         """
 
         column_name_clean = column_name + "_processed"
@@ -322,25 +343,25 @@ class CategoricalDataProcessor(BaseEstimator):
 
     def fit_transform(self, data: pd.DataFrame, column_names: list,
                       target_column: str) -> pd.DataFrame:
-        """Fits to data, then transform it
+        """Fits the data, then transforms it.
 
         Parameters
         ----------
         data : pd.DataFrame
-            data used to compute the mapping to encode the categorical
+            Data used to compute the mapping to encode the categorical
             variables with.
         column_names : list
-            Columns of data to be processed
+            Columns of data to be processed.
         target_column : str
-            Column name of the target
+            Column name of the target.
 
         Returns
         -------
         pd.DataFrame
-            data with additional transformed variables
+            Data with additional transformed variables.
         """
 
-        self.fit(data, column_names)
+        self.fit(data, column_names, target_column)
         return self.transform(data, column_names)
 
     @staticmethod
@@ -348,24 +369,27 @@ class CategoricalDataProcessor(BaseEstimator):
                               incidence: float,
                               category_size_threshold: int) -> set:
         """Fetch categories with a size below a certain threshold.
-        Note that we use an additional weighting with the overall incidence
+        Note that we use an additional weighting with the overall incidence.
 
         Parameters
         ----------
         predictor_series : pd.Series
-            Description
+            Variables data.
         incidence : float
-            global train incidence
+            Global train incidence.
         category_size_threshold : int
-            minimal size of a category to keep it as a separate category
+            Minimal size of a category to keep as a separate category.
 
         Returns
         -------
         set
-            List a categories with a count below a certain threshold
+            List a categories with a count below a certain threshold.
         """
         category_counts = predictor_series.groupby(predictor_series).size()
-        factor = max(incidence, 1 - incidence)
+        if incidence is not None:
+            factor = max(incidence, 1 - incidence)
+        else:
+            factor = 1
 
         # Get all categories with a count below a threshold
         bool_mask = (category_counts*factor) <= category_size_threshold
@@ -374,19 +398,19 @@ class CategoricalDataProcessor(BaseEstimator):
     @staticmethod
     def _replace_missings(data: pd.DataFrame,
                           column_names: Optional[list] = None) -> pd.DataFrame:
-        """Replace missing values (incl empty strings)
+        """Replace missing values (incl. empty strings).
 
         Parameters
         ----------
         data : pd.DataFrame
-            data to replace missings in
+            Data to replace missings in.
         column_names: list, optional
-            list of predictors to replace missings in
+            List of predictors to replace missings in.
 
         Returns
         -------
         list
-            list of unique values in the data
+            List of unique values in the data.
         """
         # replace missings (incl. empty string)
         regex = re.compile("^\\s+|\\s+$")
@@ -404,10 +428,14 @@ class CategoricalDataProcessor(BaseEstimator):
 
     @staticmethod
     def _compute_p_value(X: pd.Series, y: pd.Series, category: str,
+                         model_type: str,
                          scale_contingency_table: bool) -> float:
-        """Calculates p-value in contingency table (chi-square test) in
-        order to evaluate whether category of interest is significantly
-        different from the rest of the categories, given the target variable.
+        """Calculates p-value in order to evaluate whether category of
+        interest is significantly different from the rest of the
+        categories, given the target variable.
+
+        In case model_type is "classification", chi-squared test based on a contingency table.
+        In case model_type is "regression", Kruskal-Wallis test.
 
         Parameters
         ----------
@@ -416,42 +444,53 @@ class CategoricalDataProcessor(BaseEstimator):
         y : pd.Series
             Target data.
         category : str
-            Category for which we carry out the test
+            Category for which we carry out the test.
+        model_type : str
+            Model type (``classification`` or ``regression``).
         scale_contingency_table : bool
-            Whether we scale contingency table with incidence rate
+            Whether we scale contingency table with incidence rate.
+            Only used when model_type = "classification".
 
         Returns
         -------
         float
-            p-value of chi-square test
+            The p-value of applied statistical test.
         """
         df = pd.concat([X, y], axis=1)
+        df.columns = ["X", "y"]
         df["other_categories"] = np.where(X == category, 0, 1)
 
-        contigency_table = pd.crosstab(index=df['other_categories'], columns=y,
-                                       margins=False)
+        if model_type == "classification":
+            contingency_table = pd.crosstab(index=df["other_categories"], columns=df["y"],
+                                            margins=False)
 
-        # if true, we scale the "other" categories
-        if scale_contingency_table:
-            size_other_cats = contigency_table.iloc[1].sum()
-            incidence_mean = y.mean()
+            # if true, we scale the "other" categories
+            if scale_contingency_table:
+                size_other_cats = contingency_table.iloc[1].sum()
+                incidence_mean = y.mean()
 
-            contigency_table.iloc[1, 0] = (1-incidence_mean) * size_other_cats
-            contigency_table.iloc[1, 1] = incidence_mean * size_other_cats
-            contigency_table = contigency_table.values.astype(np.int64)
+                contingency_table.iloc[1, 0] = (1-incidence_mean) * size_other_cats
+                contingency_table.iloc[1, 1] = incidence_mean * size_other_cats
+                contingency_table = contingency_table.values.astype(np.int64)
 
-        return stats.chi2_contingency(contigency_table, correction=False)[1]
+            pval = stats.chi2_contingency(contingency_table, correction=False)[1]
+
+        elif model_type == "regression":
+            pval = stats.kruskal(df.y[df.other_categories == 0],
+                                 df.y[df.other_categories == 1])[1]
+
+        return pval
 
     @staticmethod
     def _replace_categories(data: pd.Series, categories: set,
                             replace_with: str) -> pd.Series:
-        """replace categories in set with "Other" and transform the remaining
-        categories to strings to avoid type errors later on in the pipeline
+        """Replace categories in set with "Other" and transform the remaining
+        categories to strings to avoid type errors later on in the pipeline.
 
         Parameters
         ----------
         data : pd.Series
-            Dataset which contains the variable to be replaced
+            Dataset which contains the variable to be replaced.
         categories : set
             Cleaned categories.
         replace_with: str
@@ -460,7 +499,7 @@ class CategoricalDataProcessor(BaseEstimator):
         Returns
         -------
         pd.Series
-            Series with replaced categories
+            Series with replaced categories.
         """
         return data.apply(
             lambda x: str(x) if x in categories else replace_with)
